@@ -1,9 +1,9 @@
-# $Id: ODBC.pm 15265 2012-04-07 13:53:28Z mjevans $
+# $Id$
 #
 # Copyright (c) 1994,1995,1996,1998  Tim Bunce
 # portions Copyright (c) 1997-2004  Jeff Urlwin
 # portions Copyright (c) 1997  Thomas K. Wenrich
-# portions Copyright (c) 2007-2012 Martin J. Evans
+# portions Copyright (c) 2007-2013 Martin J. Evans
 #
 # You may distribute under the terms of either the GNU General Public
 # License or the Artistic License, as specified in the Perl README file.
@@ -19,7 +19,7 @@ require 5.008;
 # see discussion on dbi-users at
 # http://www.nntp.perl.org/group/perl.dbi.dev/2010/07/msg6096.html and
 # http://www.dagolden.com/index.php/369/version-numbers-should-be-boring/
-$DBD::ODBC::VERSION = '1.37';
+$DBD::ODBC::VERSION = '1.45';
 
 {
     ## no critic (ProhibitMagicNumbers ProhibitExplicitISA)
@@ -32,7 +32,7 @@ $DBD::ODBC::VERSION = '1.37';
 
     @ISA = qw(Exporter DynaLoader);
 
-    # my $Revision = substr(q$Id: ODBC.pm 15265 2012-04-07 13:53:28Z mjevans $, 13,2);
+    # my $Revision = substr(q$Id$, 13,2);
 
     require_version DBI 1.609;
 
@@ -60,12 +60,28 @@ $DBD::ODBC::VERSION = '1.37';
         SQL_DIAG_ROW_NUMBER => -1248,
         SQL_DIAG_SERVER_NAME => 11,
         SQL_DIAG_SQLSTATE => 4,
-        SQL_DIAG_SUBCLASS_ORIGIN => 9
+        SQL_DIAG_SUBCLASS_ORIGIN => 9,
+        # TAF constants - these are just copies of Oracle constants
+        # events:
+        OCI_FO_END     => 0x00000001,
+        OCI_FO_ABORT   => 0x00000002,
+        OCI_FO_REAUTH  => 0x00000004,
+        OCI_FO_BEGIN   => 0x00000008,
+        OCI_FO_ERROR   => 0x00000010,
+        # callback return codes:
+        OCI_FO_RETRY   => 25410,
+        # types:
+        OCI_FO_NONE    => 0x00000001,
+        OCI_FO_SESSION => 0x00000002,
+        OCI_FO_SELECT  => 0x00000004,
+        OCI_FO_TXNAL   => 0x00000008
     };
     our @EXPORT_DIAGS = qw(SQL_DIAG_CURSOR_ROW_COUNT SQL_DIAG_DYNAMIC_FUNCTION SQL_DIAG_DYNAMIC_FUNCTION_CODE SQL_DIAG_NUMBER SQL_DIAG_RETURNCODE SQL_DIAG_ROW_COUNT SQL_DIAG_CLASS_ORIGIN SQL_DIAG_COLUMN_NUMBER SQL_DIAG_CONNECTION_NAME SQL_DIAG_MESSAGE_TEXT SQL_DIAG_NATIVE SQL_DIAG_ROW_NUMBER SQL_DIAG_SERVER_NAME SQL_DIAG_SQLSTATE SQL_DIAG_SUBCLASS_ORIGIN);
-    our @EXPORT_OK = (@EXPORT_DIAGS);
+    our @EXPORT_TAF = qw(OCI_FO_END OCI_FO_ABORT OCI_FO_REAUTH OCI_FO_BEGIN OCI_FO_ERROR OCI_FO_RETRY OCI_FO_NONE OCI_FO_SESSION OCI_FO_SELECT OCI_FO_TXNAL);
+    our @EXPORT_OK = (@EXPORT_DIAGS, @EXPORT_TAF);
     our %EXPORT_TAGS = (
-        diags => \@EXPORT_DIAGS);
+        diags => \@EXPORT_DIAGS,
+        taf => \@EXPORT_TAF);
 
     sub parse_trace_flag {
         my ($class, $name) = @_;
@@ -79,6 +95,7 @@ $DBD::ODBC::VERSION = '1.37';
         return DBI::parse_trace_flags($class, $flags);
     }
 
+    my $methods_are_installed = 0;
     sub driver{
         return $drh if $drh;
         my($class, $attr) = @_;
@@ -95,12 +112,16 @@ $DBD::ODBC::VERSION = '1.37';
             'State' => \$DBD::ODBC::sqlstate,
             'Attribution' => 'DBD::ODBC by Jeff Urlwin, Tim Bunce and Martin J. Evans',
 	    });
-        DBD::ODBC::st->install_method("odbc_lob_read");
-        # don't clear errors - IMA_KEEP_ERR = 0x00000004
-        DBD::ODBC::st->install_method("odbc_getdiagrec", { O=>0x00000004 });
-        DBD::ODBC::db->install_method("odbc_getdiagrec", { O=>0x00000004 });
-        DBD::ODBC::db->install_method("odbc_getdiagfield", { O=>0x00000004 });
-        DBD::ODBC::st->install_method("odbc_getdiagfield", { O=>0x00000004 });
+        if (!$methods_are_installed) {
+            DBD::ODBC::st->install_method("odbc_lob_read");
+            DBD::ODBC::st->install_method("odbc_rows", { O=>0x00000000 });
+            # don't clear errors - IMA_KEEP_ERR = 0x00000004
+            DBD::ODBC::st->install_method("odbc_getdiagrec", { O=>0x00000004 });
+            DBD::ODBC::db->install_method("odbc_getdiagrec", { O=>0x00000004 });
+            DBD::ODBC::db->install_method("odbc_getdiagfield", { O=>0x00000004 });
+            DBD::ODBC::st->install_method("odbc_getdiagfield", { O=>0x00000004 });
+            $methods_are_installed++;
+        }
         return $drh;
     }
 
@@ -180,6 +201,9 @@ $DBD::ODBC::VERSION = '1.37';
             odbc_driver_complete           => undef,
             odbc_batch_size                => undef,
             odbc_array_operations          => undef, # sth and dbh
+            odbc_taf_callback              => undef,
+            odbc_trace                          => undef, # dbh
+            odbc_trace_file                          => undef, # dbh
         };
     }
 
@@ -235,27 +259,29 @@ $DBD::ODBC::VERSION = '1.37';
 
 
     sub table_info {
- 	my ($dbh, $catalog, $schema, $table, $type) = @_;
+        my ($dbh, $catalog, $schema, $table, $type) = @_;
 
-	if ($#_ == 1) {
-	   my $attrs = $_[1];
-	   $catalog = $attrs->{TABLE_CAT};
-	   $schema = $attrs->{TABLE_SCHEM};
-	   $table = $attrs->{TABLE_NAME};
-	   $type = $attrs->{TABLE_TYPE};
+        if ($#_ == 1) {
+            my $attrs = $_[1];
+            $catalog = $attrs->{TABLE_CAT};
+            $schema = $attrs->{TABLE_SCHEM};
+            $table = $attrs->{TABLE_NAME};
+            $type = $attrs->{TABLE_TYPE};
  	}
+        # the following was causing a problem
+        # changing undef to '' makes a big difference to SQLTables
+        # as SQLTables has special cases for empty string calls
+        #$catalog = q{} if (!$catalog);
+        #$schema = q{} if (!$schema);
+        #$table = q{} if (!$table);
+        #$type = q{} if (!$type);
 
-	$catalog = q{} if (!$catalog);
-	$schema = q{} if (!$schema);
-	$table = q{} if (!$table);
-	$type = q{} if (!$type);
+        # create a "blank" statement handle
+        my $sth = DBI::_new_sth($dbh, { 'Statement' => "SQLTables" });
 
-	# create a "blank" statement handle
-	my $sth = DBI::_new_sth($dbh, { 'Statement' => "SQLTables" });
-
-	DBD::ODBC::st::_tables($dbh,$sth, $catalog, $schema, $table, $type)
-	      or return;
-	return $sth;
+        DBD::ODBC::st::_tables($dbh,$sth, $catalog, $schema, $table, $type)
+              or return;
+        return $sth;
     }
 
     sub primary_key_info {
@@ -493,14 +519,14 @@ $DBD::ODBC::VERSION = '1.37';
 	return $sth;
     }
 
-    sub GetTypeInfo {
-	my ($dbh, $sqltype) = @_;
-	# create a "blank" statement handle
-	my $sth = DBI::_new_sth($dbh, { 'Statement' => "SQLGetTypeInfo" });
-	# print "SQL Type is $sqltype\n";
-	_GetTypeInfo($dbh, $sth, $sqltype) or return;
-	return $sth;
-    }
+#    sub GetTypeInfo {
+#	my ($dbh, $sqltype) = @_;
+#	# create a "blank" statement handle
+#	my $sth = DBI::_new_sth($dbh, { 'Statement' => "SQLGetTypeInfo" });
+#	# print "SQL Type is $sqltype\n";
+#	_GetTypeInfo($dbh, $sth, $sqltype) or return;
+#	return $sth;
+#    }
 
     sub type_info_all {
 	my ($dbh, $sqltype) = @_;
@@ -633,13 +659,22 @@ DBD::ODBC - ODBC Driver for DBI
 
 =head1 VERSION
 
-This documentation refers to DBD::ODBC version 1.37.
+This documentation refers to DBD::ODBC version 1.45.
+
+=head1 WARNING
+
+Due to the discovery of a long standing unicode issue in DBD::ODBC the
+next development release (1.46_N) of DBD::ODBC is likely to exhibit a
+significant change in behaviour when retrieving CHAR/VARCHAR columns
+in a Unicode DBD::ODBC build. If you believe you have experienced any
+unicode issues with DBD::ODBC you are well advised to tell me now
+before it is too late.
 
 =head1 SYNOPSIS
 
   use DBI;
 
-  $dbh = DBI->connect('dbi:ODBC:DSN', 'user', 'password');
+  $dbh = DBI->connect('dbi:ODBC:DSN=mydsn', 'user', 'password');
 
 See L<DBI> for more information.
 
@@ -656,15 +691,12 @@ C<perldoc DBD::ODBC::FAQ>.
 
 =head2 Important note about the tests
 
-Please note that some tests may fail or report they are unsupported on
-this platform.  Notably Oracle's ODBC driver will fail the "advanced"
-binding tests in t/08bind2.t.  These tests run perfectly under SQL
-Server 2000. This is normal and expected.  Until Oracle fixes their
-drivers to do the right thing from an ODBC perspective, it's going to
-be tough to fix the issue.  The workaround for Oracle is to bind date
-types with SQL_TIMESTAMP.  Also note that some tests may be skipped,
-such as t/09multi.t, if your driver doesn't seem to support returning
-multiple result sets.  This is normal.
+DBD::ODBC is unlike most other DBDs in that it connects to literally
+dozens of possible ODBC Drivers. It is practically impossible for me
+to test every one and so some tests may fail with some ODBC Drivers.
+This does not mean DBD::ODBC will not work with your ODBC Driver but
+it is worth reporting any test failures on rt.cpan.org or to the
+dbi-users mailing list.
 
 =head2 DBI attribute handling
 
@@ -673,7 +705,7 @@ DBI specification.
 
 =head3 ReadOnly (boolean)
 
-DBI documents the C<ReadOnly> attribute as being settleable and
+DBI documents the C<ReadOnly> attribute as being settable and
 retrievable on connection and statement handles. In ODBC setting
 ReadOnly to true causes the connection attribute C<SQL_ATTR_ACCESS_MODE>
 to be set to C<SQL_MODE_READ_ONLY> and setting it to false will set the
@@ -1033,7 +1065,7 @@ attribute.
 Thanks to Andrew Brown for the original patch.
 
 DBD developer note: Here lies a bag of worms. Firstly, SQL_ROWSET_SIZE
-is an ODBC 2 attribute and is usally a statement attribute not a
+is an ODBC 2 attribute and is usually a statement attribute not a
 connection attribute. However, in ODBC 2.0 you could set statement
 attributes on a connection handle and it acted as a default for all
 subsequent statement handles created under that connection handle. If
@@ -1183,6 +1215,13 @@ README.unicode file for further details.
 After calling the connect method this will be the ODBC driver's
 out connection string - see documentation on SQLDriverConnect.
 
+Typically, applications (like MS Access and many others) which build a connection string via
+dialogs and possibly SQLBrowseConnect eventually end up with a successful ODBC connection
+to the ODBC driver and database. The odbc_out_connect_string provides a string
+which you can pass to SQLDriverConnect (DBI's connect prefixed with dbi:ODBC:") which
+will connect you to the same datasource at a later date. You may also want to see
+L</odbc_driver_complete>.
+
 =head3 odbc_version
 
 This was added prior to the move to ODBC 3.x to allow the caller to
@@ -1261,12 +1300,137 @@ When set to 0 array operations are used no matter what. I know this is
 slightly counter intuitive but I've found it difficult to change the
 name (it got picked up and used in a few places very quickly).
 
+=head3 odbc_taf_callback
+
+NOTE: this is experimental until I at least see more than one ODBC
+driver which supports TAF.
+
+Transparent Application Failover (TAF) is a feature in OCI that
+allows for clients to automatically reconnect to an instance in the
+event of a failure of the instance. The reconnect happens
+automatically from within the OCI (Oracle Call Interface)
+library.
+
+TAF supports a callback function which once registered is called
+by the driver to let you know what is happening and which allows
+you to a degree, to control how the failover is handled.
+
+You need to set up TAF on your instance first and that process
+is beyond the scope of this document. Once TAF is enabled you simply
+set C<odbc_taf_callback> to a code reference which should look like
+this:
+
+  sub taf_handler {
+   my ($dbh, $event, $type) = @_;
+   # do something here
+  }
+
+DBD::ODBC will pass the connection handle ($dbh), the Oracle event
+type (OCI_FO_END, OCI_FO_ABORT, OCI_FO_REAUTH, OCI_FO_BEGIN,
+OCI_FO_ERROR) and the Oracle type (OCI_FO_NONE, OCI_FO_SESSION,
+OCI_FO_SELECT, OCI_FO_TXNAL).  Consult the Oracle documentation for
+what these are. You can import these constants using the :taf export
+tag. If your instance is not TAF enabled it is likely an attempt to
+register a callback will fail but this is driver dependent (all
+DBD::ODBC does is make a SQLSetConnectAttr call and provide a C
+wrapper which calls your Perl subroutine).
+
+Here is a commented example:
+
+  my $h = DBI->connect('dbi:ODBC:oracle','xxx','yyy',
+                       {RaiseError => 1,
+                        odbc_taf_callback => \&taf_handler}) or die "connect";
+  while (1) {
+      my $s = $h->selectall_arrayref(q/select 1 from dual/);
+      sleep 5;
+  }
+
+  sub taf_handler {
+     my ($dbh, $event, $type) = @_;
+
+     #print "taf_handler $dbh, $event, $type\n";
+
+     if ($event == OCI_FO_BEGIN) {
+         print "Instance unavailable, stand by\n";
+         print "Your TAF type is : ",
+             ($type == OCI_FO_NONE ? "NONE" :
+                  ($type == OCI_FO_SESSION ? "SESSION" :
+                       ($type == OCI_FO_SELECT ? "SELECT" : "?"))) , "\n";
+         # start a counter and each time OCI_FO_ERROR is passed in we will
+         # count down and abort the failover when we hit 0.
+         $count = 10;
+         return 0;
+     } elsif ($event == OCI_FO_ERROR) {
+         # We get an OCI_FO_ERROR each time the failover fails
+         # sleep a while until the count hits 0
+         if (--$count < 1) {
+             print "Giving up\n";
+             return 0;            # give up
+         } else {
+             print "Retrying...\n";
+             sleep 1;
+             return OCI_FO_RETRY; # tell Oracle to retry
+         }
+     } elsif ($event == OCI_FO_REAUTH) {
+         print "Failed over user. Resuming Services\n";
+     } elsif ($event == OCI_FO_END) {
+         print "Failover ended - resuming\n";
+     }
+     return 0;
+  }
+
+NOTE: The above example is for use with the Easysoft Oracle ODBC
+Driver. ODBC does not define any standard way of supporting TAF and so
+different drivers may use different connection attributes to set it up
+or may even pass the callback different arguments. Unfortunately, I
+don't have access to any other ODBC driver which supports TAF. Until I
+see others I cannot create a generic interface. I'll happily accept
+patches for any other driver or if you send me a working copy of the
+driver and the documentation I will add support for it.
+
+=head3 odbc_trace_file
+
+Specify the name and path to a file you want ODBC API trace
+information to be written to. See L</odbc_trace>.
+
+=head3 odbc_trace
+
+Enable or disable ODBC API tracing. Set to 1 to enable and 0 to
+disable.
+
+This calls SQLSetConnectAttr for SQL_ATTR_TRACE and either sets
+SQL_OPT_TRACE_ON or SQL_OPT_TRACE_OFF. Enabling tracing will tell
+the ODBC driver manager to write and ODBC API trace to the file
+named with L</odbc_trace_file>.
+
+NOTE: If you don't set odbc_trace_file most ODBC Driver Managers write
+to a file called SQL.LOG in the root directory (but this depends on
+the driver manager used).
+
+NOTE: This tracing is produced by the ODBC Driver Manager and has
+nothing to do with DBD::ODBC other than it should trace the ODBC
+calls DBD::ODBC makes i.e., DBD::ODBC is not responsible for the
+tracing mechanism itself.
+
+NOTE: Enabling tracing will probably slow your application down a lot.
+I'd definitely think twice about it if in a production environment
+unless you are desperate as it tends to produce very large trace
+files for short periods of ODBC activity.
+
 =head2 Private statement attributes
 
 =head3 odbc_more_results
 
 Use this attribute to determine if there are more result sets
-available.  SQL Server supports this feature.  Use this as follows:
+available.
+
+Any ODBC Driver which batches results or counts of inserts/updates
+will need you to loop on odbc_more_results until there are no more
+results. e.g., if you are performing multiple selects in a procedure or
+multiple inserts/updates/deletes then you will probably need to loop on
+odbc_more_results.
+
+Use odbc_more_results as follows:
 
   do {
      my @row;
@@ -1279,11 +1443,40 @@ Note that with multiple result sets and output parameters (i.e,. using
 bind_param_inout), don't expect output parameters to written to until ALL
 result sets have been retrieved.
 
+Under the hood this attribute causes a call to the ODBC API
+SQLMoreResults and then any result set, insert/update/delete or output
+parameters are described by DBD::ODBC and the statement handle will be
+ready for processing the new result.
+
 =head2 Private statement methods
 
+=head3 odbc_rows
+
+This method was added in 1.42_1.
+
+In 64 bit ODBC SQLRowCount can return a 64bit value for the number of
+rows affected. Unfortunately, the DBI DBD interface currently (at
+least until 1.622) defines execute as returning an int so values which
+cannot fit in an int are truncated. See RT 81911.
+
+From DBD::ODBC 1.42_1 DBD::ODBC
+
+o defines this method which will return the affected rows in an IV
+(and IVs are guaranteed to be able to hold a pointer) so you
+can get the real affected rows without truncation.
+
+o if it detects an overflow in the execute method it will issue
+a warning (if Warn is on which it is by default) and return INT_MAX.
+
+At some stage DBI may change to fix the issue this works around.
+
+NOTE: the return from odbc_rows is not the raw value returned by
+SQLRowCount. It is the same as execute normally returns e.g., 0E0 (for
+0), -1 for unknown and N for N rows affected where N > 0.
+
 =head3 odbc_lob_read
 
-  $chrs_or_bytes_read = $sth->lob_read($column_no, \$lob, $length, \%attr);
+  $chrs_or_bytes_read = $sth->odbc_lob_read($column_no, \$lob, $length, \%attr);
 
 Reads C<$length> bytes from the lob at column C<$column_no> returning
 the lob into C<$lob> and the number of bytes or characters read into
@@ -1303,7 +1496,7 @@ unicode support. C<$chrs_or_bytes_read> will by the bytes read when
 the column types SQL_C_CHAR or SQL_C_BINARY are used and characters
 read if the column type is SQL_C_WCHAR.
 
-When built with unicode support C<$length> specifes the amount of
+When built with unicode support C<$length> specifies the amount of
 buffer space to be used when retrieving the lob data but as it is
 returned as SQLWCHAR characters this means you at most retrieve
 C<$length/2> characters. When those retrieved characters are encoded
@@ -1331,7 +1524,7 @@ NOTE: You can retrieve only part of a lob but you will probably have
 to call finish on the statement handle before you do anything else
 with that statement. When only retrieving part of a large lob you
 could see a small delay when you call finish as some protocols used
-by ODBC drivers send the lob down the socket synchonously and there is
+by ODBC drivers send the lob down the socket synchronously and there is
 no way to stop it (this means the ODBC driver needs to read all the
 lob from the socket even though you never retrieved it all yourself).
 
@@ -1361,10 +1554,11 @@ which returns the C<SQL_DRIVER_NAME>.
 This function returns a scalar value, which can be a numeric or string
 value depending on the information value requested.
 
-=head3 SQLGetTypeInfo
+=head3 GetTypeInfo
 
 B<This private function is now superceded by DBI's type_info and
-type_info_all methods.>
+type_info_all methods however as it is used by those methods it
+still exists.>
 
 This function maps to the ODBC SQLGetTypeInfo API and the argument
 should be a SQL type number (e.g. SQL_VARCHAR) or
@@ -1513,16 +1707,10 @@ use DBI's NAME and NAME_xx attributes for portability.
 
 =head3 DescribeCol
 
-B<This private function is now superceded by DBI's statement attributes
-NAME, TYPE, PRECISION, SCALE, NULLABLE etc).>
+Removed in DBD::ODBC 1.40_3
 
-See the ODBC specification for the SQLDescribeCol API.
-You call SQLDescribeCol like this:
-
-  @info = $sth->func($column, "DescribeCol");
-
-The returned array contains the column attributes in the order described
-in the ODBC specification for SQLDescribeCol.
+Use the DBI's statement attributes NAME, TYPE, PRECISION, SCALE,
+NULLABLE etc instead.
 
 =head2 Additional bind_col attributes
 
@@ -1539,8 +1727,11 @@ DiscardString to the prepare attributes then if the returned bound
 data is capable of being converted to that type the scalar's pv (the
 string portion of a scalar) is cleared.
 
+NOTE: post DBD::ODBC 1.37, DBD::ODBC binds all SQL_INTEGER columns as
+SQL_C_LONG and DiscardString is irrelevant.
+
 This is especially useful if you are using a module which uses a
-scalars flags and/or pv to decide if a scalar is a number. JSON::XS
+scalar's flags and/or pv to decide if a scalar is a number. JSON::XS
 does this and without this flag you have to add 0 to all bound column
 data returning numbers to get JSON::XS to encode it is N instead of
 "N".
@@ -1559,10 +1750,10 @@ performed an error will be raised.
 
 This is probably not a lot of use with DBD::ODBC as if you ask for say
 an SQL_INTEGER and the data is not able to be converted to an integer
-the ODBC driver will problably return "Invalid character value for
+the ODBC driver will probably return "Invalid character value for
 cast specification (SQL-22018)".
 
-NOTE: For DiscardString you need at least DBI 1.611.
+NOTE: For StrictlyTyped you need at least DBI 1.611.
 
 =head3 TreatAsLOB
 
@@ -1578,7 +1769,7 @@ strings (after modification) passed to the prepare and do methods.
 
 From DBI 1.617 DBI also defines ENC (encoding), CON (connection) TXN
 (transaction) and DBD (DBD only) trace flags. DBI's ENC and CON trace
-flags are synonomous with DBD::ODBC's odbcunicode and odbcconnection
+flags are synonymous with DBD::ODBC's odbcunicode and odbcconnection
 trace flags though I may remove the DBD::ODBC ones in the
 future. DBI's DBD trace flag allows output of only DBD::ODBC trace
 messages without DBI's trace messages.
@@ -1735,7 +1926,7 @@ The DBI specification post 1.608 says in bind_param:
   The data type is 'sticky' in that bind values passed to execute()
   are bound with the data type specified by earlier bind_param()
   calls, if any.  Portable applications should not rely on being able
-  to change the data type after the first C<bind_param> call.
+  to change the data type after the first bind_param call.
 
 DBD::ODBC does allow a parameter to be rebound with another data type as
 ODBC inherently allows this. Therefore you can do:
@@ -1768,6 +1959,22 @@ when inserting, updating or deleting many rows in one go. Note,
 execute_array uses execute_for_fetch when the parameters are passed
 for column-wise binding.
 
+NOTE: DBD::ODBC 1.34_1 to DBD::ODBC 1.36_1 set the default to use
+DBD::ODBC's own execute_for_fetch but quite a few ODBC drivers just
+cannot handle it. As such, from DBD::ODBC 1.36_2 the default was
+changed to not use DBD::ODBC's execute_for_fetch (i.e., you need to
+enable it with odbc_array_operations).
+
+NOTE: Some ODBC drivers don't support setting SQL_ATTR_PARAMSET_SIZE >
+1, and hence cannot support binding arrays of parameters. The only way
+to detect this is to attempt to set SQL_ATTR_PARAMSET_SIZE to a value
+greater than 1 and it is too late once someone has called
+execute_for_fetch. I don't want to add test code on each connect to
+test for this as it will affect everyone, even those not using the
+native execute_for_fetch so for now it is a suck it and see. For your
+information MS Access which does not support arrays of parameters
+errors with HY092, "Invalid attribute/option identifier".
+
 However, there are a small number of differences between using
 DBD::ODBC's execute_for_fetch compared with using DBI's default
 implementation (which simply calls execute repeatedly once per row).
@@ -1833,11 +2040,39 @@ DBI's inbuilt execute_* methods. See rt 75687.
 =head3 type_info_all
 
 Many ODBC drivers now return 20 columns in type_info_all rather than
-the 19 DBI documents DBI documents. The 20th column is usually called
+the 19 DBI documents. The 20th column is usually called
 "USERTYPE".  Recent MS SQL Server ODBC drivers do this. Fortunately
 this should not adversely affect you so long as you are using the keys
 provided at the start of type_info_all.
 
+=head3 Binding Columns
+
+The DBI specification allows a column type to be overridden in the call
+to the bind_col method. Mostly, DBD::ODBC ignores this type as it
+binds integers (SQL_INTEGER) as a SQL_C_LONG (since DBD::ODBC 1.38_1)
+and all other columns as SQL_C_CHAR or SQL_C_WCHAR and it is too late
+to change the bind type after the result-set has been described
+anyway. The only time when the TYPE passed to bind_col is used in
+DBD::ODBC is when it is SQL_NUMERIC or SQL_DOUBLE in which case
+DBD::ODBC will call DBI's sql_type_cast method.
+
+Since DBD::ODBC 1.38_1 if you attempt to change the bind type after
+the column has already bound DBD::ODBC will issue a warning and ignore
+your column type change e.g.,
+
+  my $s = $h->prepare(q/select a from mytable);
+  $s->execute;  # The column type was determined here
+  my $r;
+  $s->bind_col(1, \$r); # and bound as the right type here
+  $s->execute;
+  $s->bind_col(1, \$r, {TYPE => SQL_XXX}); # warning, type changed
+
+Basically, if you are passing a TYPE to bind_col with DBD::ODBC (other
+than SQL_NUMERIC or SQL_DOUBLE) your code is probably wrong.
+
+Significant changes occurred in DBD::ODBC at 1.38_1 for binding
+columns. Please see the Changes file.
+
 =head2 Unicode
 
 The ODBC specification supports wide character versions (a postfix of
@@ -1845,8 +2080,11 @@ The ODBC specification supports wide character versions (a postfix of
 character version of SQLDriverConnect.
 
 In ODBC on Windows the wide characters are defined as SQLWCHARs (2
-bytes) and are UCS-2. On non-Windows, the main driver managers I know
-of have implemented the wide character APIs differently:
+bytes) and are UCS-2 (but UTF-16 is accepted by some drivers now e.g.,
+MS SQL Server 2012 and the new collation suffix _SC which stands for
+Supplementary Character Support). On non-Windows, the main driver
+managers I know of have implemented the wide character APIs
+differently:
 
 =over
 
@@ -1854,7 +2092,7 @@ of have implemented the wide character APIs differently:
 
 unixODBC mimics the Windows ODBC API precisely meaning the wide
 character versions expect and return 2-byte characters in
-UCS-2.
+UCS-2 or UTF-16.
 
 unixODBC will happily recognise ODBC drivers which only have the ANSI
 versions of the ODBC API and those that have the wide versions
@@ -1951,7 +2189,7 @@ As of DBD::ODBC 1.32_3 meta data calls accept Unicode strings.
 Since version 1.16_4, the default parameter bind type is SQL_WVARCHAR
 for unicode builds of DBD::ODBC. This only affects ODBC drivers which
 do not support SQLDescribeParam and only then if you do not
-specifically set a sql type on the bind_param method call.
+specifically set a SQL type on the bind_param method call.
 
 The above Unicode support has been tested with the SQL Server, Oracle
 9.2+ and Postgres drivers on Windows and various Easysoft ODBC drivers
@@ -2036,14 +2274,12 @@ API currently uses UCS-2 it does not support Unicode characters with
 code points above 0xFFFF (if you know better I'd like to hear from
 you). However, because DBD::ODBC uses UTF-16 encoding you can still
 insert Unicode characters above 0xFFFF into your database and retrieve
-them back correctly but they will not being treated as a single
+them back correctly but they may not being treated as a single
 Unicode character in your database e.g., a "select length(a_column)
-from table" with a single Unicode character above 0xFFFF will most
-likely return 2 and not 1 so you cannot use database functions on that
+from table" with a single Unicode character above 0xFFFF may
+return 2 and not 1 so you cannot use database functions on that
 data like upper/lower/length etc but you can at least save the data in
-your database and get it back. This is a fudge and I cannot say I'm
-overjoyed by it but it is what the majority of people who use
-DBD::ODBC have requested.
+your database and get it back.
 
 When built for unicode, DBD::ODBC will always call SQLDriverConnectW
 (and not SQLDriverConnect) even if a) your connection string is not
@@ -2132,44 +2368,23 @@ SQL type to the end of the C<bind_param> method.
 
 =head2 Version Control
 
-DBD::ODBC source code is under version control at svn.perl.org.  If
-you would like to use the "bleeding" edge version, you can get the
-latest from svn.perl.org via Subversion version control.  Note there
-is no guarantee that this version is any different than what you get
-from the tarball from CPAN, but it might be :)
-
-You may read about Subversion at L<http://subversion.tigris.org>
-
-You can get a subversion client from there and check dbd-odbc out via:
-
-   svn checkout http://svn.perl.org/modules/dbd-odbc/trunk <your directory name here>
-
-Which will pull all the files from the subversion trunk to your
-specified directory. If you want to see what has changed since the
-last release of DBD::ODBC read the Changes file or use "svn log" to
-get a list of checked in changes.
+DBD::ODBC source code was under version control at svn.perl.org
+until April 2013 when svn.perl.org was closed down and it is now on
+github at https://github.com/perl5-dbi/DBD-ODBC.git.
 
 =head2 Contributing
 
-There are seven main ways you may help with the development and
+There are a number of ways you may help with the development and
 maintenance of this module:
 
 =over
 
 =item Submitting patches
 
-Please use Subversion (see above) to get the latest version of
-DBD::ODBC from the trunk and submit any patches against that.
+Please send me a git pull request or email a unified diff.
 
-Please, before submitting a patch:
-
-   svn update
-   <try and included a test which demonstrates the fix/change working>
-   <test your patch>
-   svn diff > describe_my_diffs.patch
-
-and send the resulting file to me and cc the dbi-users@perl.org
-mailing list (if you are not a member - why not!).
+Please try and include a test which demonstrates the fix/change
+working.
 
 =item Reporting installs
 
@@ -2333,6 +2548,10 @@ L<http://www.easysoft.com/developer/languages/perl/multiple-active-statements.ht
 
 L<http://www.easysoft.com/developer/interfaces/odbc/64-bit.html>
 
+How do I insert Unicode supplementary characters into SQL Server from Perl?
+
+L<http://www.easysoft.com/support/kb/kb01043.html>
+
 =head2 Frequently Asked Questions
 
 Frequently asked questions are now in L<DBD::ODBC::FAQ>. Run
@@ -2386,4 +2605,13 @@ Jeff Urlwin and Martin J. Evans - see the source.
 
 L<DBI>
 
+DBD::ODBC can be used with many ODBC drivers to many different
+databases.  If you want a generic DBD for multiple databases DBD::ODBC
+is probably for you.  If you are only accessing a single database then
+you might want to look for DBD::my_database (e.g. DBD::Oracle) as
+database specific DBDs often have more functionality.
+
+L<DBIx::LogAny> or L<DBIx::Log4perl> for logging DBI method calls, SQL,
+parameters and results.
+
 =cut
